@@ -1,9 +1,9 @@
 //
-// Copyright (c) 2015-2019 CNRS INRIA
+// Copyright (c) 2015-2020 CNRS INRIA
 //
 
-#ifndef __pinocchio_python_algorithm_hpp__
-#define __pinocchio_python_algorithm_hpp__
+#ifndef __pinocchio_python_algorithms_hpp__
+#define __pinocchio_python_algorithms_hpp__
 
 #include "pinocchio/bindings/python/fwd.hpp"
 #include <boost/python.hpp>
@@ -34,6 +34,7 @@ namespace pinocchio
     void exposeRNEADerivatives();
     void exposeABADerivatives();
     void exposeKinematicsDerivatives();
+    void exposeFramesDerivatives();
     void exposeCentroidalDerivatives();
 
     void exposeAlgorithms();
@@ -41,5 +42,5 @@ namespace pinocchio
   } // namespace python
 } // namespace pinocchio
 
-#endif // ifndef __pinocchio_python_algorithm_hpp__
+#endif // ifndef __pinocchio_python_algorithms_hpp__
 
